	/*----------------------------------|
	|    Trycom Test Program Utility    |
	|                                   |
	|    Last update: 06/30/2005        |
	|----------------------------------*/

#include   <dos.h>
#include   <stdio.h>
#include   <conio.h>
#include   <io.h>
#include   <stdarg.h>
#include   <string.h>

#define	fcr  2  	   /* point to UART FIFO control REG. offset addr. */
#define	lcr  3  	   /* point to UART line control REG. offset addr. */
#define	mcr  4             /* point to UART modem control REG. offset addr. */
#define	lsr  5  	   /* point to UART line status REG. offset addr. */
#define	ier  1             /* point to UART interrupt enable REG. offset addr. */
#define	lsb  0             /* point to UART LSB REG. offset addr. */
#define	msb  1             /* point to UART MSB REG. offset addr. */
#define uart_lcr 3	   /* For LCR REG. (length = 8 bits, no stop bit, none parity) */
#define _cr  0xd                      /* ENTER key */
#define ESC  0x1b

int	init_com(void) ;
void	uart_direction( int direction ) ;
int 	send_byte( unsigned char byte ) ;
unsigned char receive_byte(unsigned char *byte );


int	com_addr = 0x3f8 ;	   /* COM port base I/O Addr. */
int	true  = 0      ;
int     false = 1      ;
int     _com_port = 1  ;              /* Set default COM port number */
int     i;
unsigned char   _command[20] ;

void main()
{

/*      Initial COM port */
	(void) init_com() ;
/*      Send Command */
	while(1)
	{
	  printf("\nInput Command : ");
	  scanf("%s",&_command);       /* Get Command */
	  _command[strlen(_command)] = _cr;
	  if(send_cmd() == 0)
	   {
	     delay(100);
	     if(!rcv_cmd())
			   printf("\nReceived Message : %s",_command);
		       else
			   printf("\nNo Respond...");
	     }
	   else
	     printf("\nCom Port Fail !");

	  printf("\nPress ESC For Exit");
	  if(getch() == ESC) exit(0);
	}
}
/*===== End of main =====================================================================*/

int  init_com()
{
	int	i ;
	unsigned char _lsb     ;     	/* baud rate LSB value */
	unsigned char _msb = 0 ;     	/* baud rate MSB value */

	_lsb = 0x0c ;                   /* Baud rate = 9600    */

	com_addr = peek( 0x40, 0 ) ;    /* Get COM-1 port base I/O addr. */

	outportb( com_addr+lcr,0x80 );           /* Point to DLAB REG. */
	outportb( com_addr+lsb,_lsb ) ;          /* Write baud rate to LSB REG. */
	outportb( com_addr+msb,_msb ) ;          /* Write baud rate to MSB REG. */
	outportb( com_addr+lcr,(3 & 0x1f) );     /* Initial LCR REG.  */
	outportb( com_addr+ier,0 ) ;             /* Disable interrup */
	outportb( com_addr+mcr,0 ) ;             /* Clear DTR,RTS for set RS-485 to Receive Mode*/
	outportb( com_addr+fcr,0x7 ) ;           /* Clear I/O FIFO REG. */
	for (i=0; i<8; i++) inportb(com_addr) ;	 /* Clear Shift REG. */
	outportb( com_addr+fcr,0x1 ) ;           /* Enable I/O FIFO REG. */
	return( true ) ;
}
/*----- End of init_com() ---------------------------------------------------------------*/


/*--------------------------------------------
  Send Command to COM port
  return : = 0 - pass
	   = 1 - Send fail
---------------------------------------------*/
int send_cmd()
{
     int i ;

     /* Send command to COM port */
     for( i=0 ;; i++ )
     {
	if( send_byte( _command[i] ) != 0 )
	{
	    return(1) ;  /* Command send error  */
	}

	if( _command[i] == _cr )
	{
	   return(0) ; /* Command send OK */
	}

     }
}
/*--------------------------------------------------------------------------------------*/


/*--------------------------------------------
  receive command Response from COM port
  return : = 0 - pass
	   = 1 - receive fail
---------------------------------------------*/
int rcv_cmd()
{
     int i,s=0 ;
     int retry_times = 5 ;
     for( i=0 ; i < retry_times ; )
     {
	_command[s] = 0 ; /* clear */
	if(receive_byte(&_command[s]) == 0 )
	{
	   if( _command[s] == _cr )
	   {   _command[s] = 0 ;  /* change (cr) code to 0 for display response string */
	       return(0) ;        /* receive OK */
	   }
	   s++ ;
	   i = 0 ;
	}
	else
	       i++ ;
     }
     return(1)  ;  /* Receive Timeout */
}
/*--------------------------------------------------------------------------------------*/





/*----------------------------------------------
 Send character

 Entry  : character to send

 Return : 0 - Successful
	  1 - Time out
------------------------------------------------*/
int send_byte( unsigned char byte )
{	unsigned int  i,s ;
	int  retry_times = 15 ;


	for (i=0 ; i <= retry_times ;i++ )
	{
	     if( (0x20&(inportb(com_addr+lsr)) ) == 0x20 )
	     {
		outportb( com_addr, byte ) ;     /* Send byte to UART */
		for (s=0 ; s <= retry_times;)
		{
		     if( (0x60&(inportb(com_addr+lsr))) == 0x60 )  return(0) ;
		}
		return(1)  ;
	     }
	  delay(10);
	}
	return(1) ;
}
/*------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------
 Receive character

 Entry  : unsigned byte pointer for receive byte

 Return : 0 - Successful
	  1 - Time out
---------------------------------------------------------------------------*/
unsigned char receive_byte(unsigned char *byte )
{	int  i ;
	int	retry_times = 15 ;

	for (i=0 ; i <= retry_times ;i++)
	{
	     if( (inportb(com_addr+lsr )&0x1) == 0x1)
	     {
		*byte = inportb( com_addr ) ;     /* receive byte from UART */
		return(0) ;
	     }
	  delay(10);
	}
	return(1) ;
}
/*------------------------------------------------------------------------*/